package net.wanji.opt.synthesis.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.druid.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.opt.dao.mapper.StrategyControlInfoMapper;
import net.wanji.opt.po.StrategyControlDataEntity;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.synthesis.service.PushStrategyControlService;
import net.wanji.opt.synthesis.service.StrategyControlService;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author duanruiming
 * @date 2024/11/03 17:37
 */
@Slf4j
@Service
public class StrategyControlServiceImpl implements StrategyControlService {

    @Resource
    private PushStrategyControlService pushStrategyControlService;
    @Resource
    private StrategyControlInfoMapper strategyControlInfoMapper;

    @Override
    public JsonViewObject strategyInfoOperation(StrategyControlVO strategyControlVO) throws Exception {

        try {
            // 存库
            List<StrategyControlDataEntity> entities = new ArrayList<>();
            List<StrategyControlDataVO> dataList = strategyControlVO.getDataList();
            for (StrategyControlDataVO dataVO : dataList) {
                StrategyControlDataEntity entity = new StrategyControlDataEntity();
                BeanUtils.copyProperties(entity, dataVO);
                entity.setTime(JacksonUtils.getInstance().writeValueAsString(dataVO.getTime()));
            }
            strategyControlInfoMapper.insertBatch(entities);

            StrategyControlReq req = convertReq(strategyControlVO);
            // todo 测试数据
            //req.setData(Arrays.asList(detailData));
            Result result = pushStrategyControlService.push(req);
            if (Objects.nonNull(result)) {
                JsonViewObject jsonViewObject = JsonViewObject.newInstance();
                jsonViewObject.setCode(Integer.valueOf(result.getCode()));
                jsonViewObject.setMessage(result.getMsg());
                jsonViewObject.setContent(result.getData());
                jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
                return jsonViewObject;
            } else {
                return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
            }
        } catch (Exception e) {
            log.error("策略操作失败：{}", e);
            throw new Exception(e);
        }
    }

    private StrategyControlReq convertReq(StrategyControlVO vo) throws Exception {
        try {
            List<StrategyControlDataVO> dataList = vo.getDataList();
            StrategyControlReq req = new StrategyControlReq();
            if (!CollectionUtils.isEmpty(dataList)) {
                List<StrategyControlDataReq> dataReqList = new ArrayList<>(dataList.size());
                for (StrategyControlDataVO dataVO : dataList) {
                    StrategyControlDataReq detailData = new StrategyControlDataReq();
                    detailData.setBiz_id(dataVO.getBizId());
                    detailData.setBiz_type(dataVO.getBizType());
                    detailData.setStrategy(dataVO.getStrategy());
                    detailData.setSchedule_start(DateUtil.format(dataVO.getScheduleStart(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    detailData.setSchedule_end(DateUtil.format(dataVO.getScheduleEnd(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    List<StrategyControlDataVO.TimeTable> timeTableList = dataVO.getTime();
                    List<StrategyControlDataReq.Time_table> timeTables = new ArrayList<>(timeTableList.size());
                    timeTableList.forEach(item -> {
                        StrategyControlDataReq.Time_table timeTable = new StrategyControlDataReq.Time_table();
                        timeTable.setTime_list(item.getTimeList());
                        timeTable.setWeek(item.getWeek());
                        timeTables.add(timeTable);
                    });
                    detailData.setTime(timeTables);
                    detailData.setFreq(dataVO.getFrequency());
                    detailData.setStatus(dataVO.getStatus());
                    detailData.setAction(dataVO.getAction());
                    dataReqList.add(detailData);
                    req.setData(dataReqList);
                }
            }
            return req;
        } catch (Exception e) {
            log.error("策略控制操作数据转换异常：{}", e);
            throw new Exception(e);
        }
    }


    public static void main(String[] args) throws Exception {

        StrategyControlDataReq detailData = new StrategyControlDataReq();
        detailData.setBiz_id("241940");
        detailData.setBiz_type(0);
        detailData.setStrategy(3);
        Date date = new Date();
        detailData.setSchedule_start(DateUtil.format(date, DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
        detailData.setSchedule_end(DateUtil.format(date, DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
        StrategyControlDataReq.Time_table timeTable = new StrategyControlDataReq.Time_table();
        timeTable.setWeek(1);
        timeTable.setTime_list(new String[]{"00:00-01:00", "01:00-02:00"});
        StrategyControlDataReq.Time_table timeTable1 = new StrategyControlDataReq.Time_table();
        timeTable1.setWeek(2);
        timeTable1.setTime_list(new String[]{"02:00-03:00", "03:00-04:00"});
        detailData.setTime(Arrays.asList(timeTable1, timeTable));
        detailData.setFreq(1000 * 60);
        detailData.setStatus(1);
        detailData.setAction("insert");

        // todo 测试数据
        StrategyControlReq req = new StrategyControlReq();
        req.setData(Arrays.asList(detailData));
        System.err.println(JacksonUtils.getInstance().writeValueAsString(req));


        System.err.println(new Date());
    }
}
